@@ -1,101 +1,10 @@
 
 =head1 NAME
 
-DBD::XBase - DBI driver for XBase compatible db files
-
-=head1 SYNOPSIS
-
-    use DBI;
-    my $dbh = DBI->connect("DBI:XBase:/directory/subdir")
-    				or die $DBI::errstr;
-    my $sth = $dbh->prepare("select MSG from test where ID != 1")
-    				or die $dbh->errstr();
-    $sth->execute() or die $sth->errstr();
-
-    my @data;
-    while (@data = $sth->fetchrow_array())
-		{
-		## further processing
-		}
-
-=head1 DESCRIPTION
-
-DBI compliant driver for module XBase. Please refer to DBI(3)
-documentation for how to actually use the module.
-In the I<connect> call, specify the directory for a database name.
-This is where the DBD::XBase will look for the tables.
-
-The SQL commands currently supported by DBD::XBase include:
-
-=over 4
-
-=item select
-
-    select fields from table [ where condition ]
-
-Fields is a comma separated list of fields or a C<*> for all. The
-C<where> condition specifies which rows will be returned, you can
-compare fields and constants and stack expressions using C<and> or
-C<or>, and also use brackets. Examples:
-
-    select * from salaries where name = "Smith"	
-    select first,last from people where login = "ftp"
-						or uid = 1324
-
-=item delete
-
-    delete from table [ where condition ]
-
-The C<where> condition si the same as for C<select>. Examples:
-
-    delete from jobs		## emties the table
-    delete from jobs where companyid = "ISW"
-
-=item insert
-
-    insert into table [ ( fields ) ] values ( list of values )
-
-Here fields is a comma separated list of fields to set, list of values
-is a list of values to assign. If the fields are not specified, the
-fields in the natural order in the table are set. Example:
-
-    insert into accounts (login, uid) values ("guest", 65534)
-
-=item update
-
-    update table set field = new value [ , set more fields ]
-					[ where condition ]
-
-Example:
-
-    update table set uid = 65534 where login = "guest"
-
-=item create table
-
-    create table table name ( columns specification )
-
-Columns specification is a comma separated list of column names and
-types. Example:
-
-    create table rooms ( roomid int, category int, balcony boolean)
-
-=over
-
-=head1 VERSION
-
-0.058
-
-=head1 AUTHOR
-
-(c) Jan Pazdziora, adelton@fi.muni.cz
-
-=head1 SEE ALSO
-
-perl(1), DBI(3), XBase(3)
+DBD::XBase - DBI driver for XBase compatible database files
 
 =cut
 
-
 # ##################################
 # Here starts the DBD::XBase package
 
@@ -110,7 +19,7 @@ use vars qw($VERSION @ISA @EXPORT $err $errstr $drh);
 
 require Exporter;
 
-$VERSION = '0.058';
+$VERSION = '0.063';
 
 $err = 0;
 $errstr = '';
@@ -228,6 +137,19 @@ sub STORE {
 	$dbh->DBD::_::db::STORE($attrib, $value);
 	}
 
+sub _ListTables
+	{
+	my $dbh = shift;
+	opendir DIR, $dbh->{'dsn'} or return;
+	my @result = ();
+	while (defined(my $item = readdir DIR))
+		{
+		next unless $item =~ s/\.dbf$//;
+		push @result, $item;
+		}
+	closedir DIR;
+	@result;
+	}
 
 package DBD::XBase::st;
 use strict;
@@ -332,14 +254,10 @@ sub fetch
         my $sth = shift;
 	my $current = $sth->{'xbase_current_record'};
 	my $parsed_sql = $sth->{'xbase_parsed_sql'};
+	my $table = $sth->{'xbase_table'};
 	return unless $parsed_sql->{'command'} eq 'select';
 	$current = 0 unless defined $current;
-	my $table = $sth->{'xbase_table'};
-	my @fields;
-	if (defined $parsed_sql->{'selectall'})
-		{ @fields = $table->field_names(); }
-	else
-		{ @fields = @{$parsed_sql->{'selectfields'}}; }
+	my @fields = @{ $sth->FETCH('NAME') };
 	while ($current <= $table->last_record())
 		{
 		my $values = $table->get_record_as_hash($current);
@@ -353,4 +271,114 @@ sub fetch
 	}
 *fetchrow_arrayref = \&fetch;
 
+sub FETCH
+	{
+	my $sth = shift;
+	my $parsed_sql = $sth->{'xbase_parsed_sql'};
+	my $table = $sth->{'xbase_table'};
+	if (defined $_[0] and $_[0] eq 'NAME')
+		{
+		if (defined $parsed_sql->{'selectall'})
+			{ return [ $table->field_names() ]; }
+		else
+			{ return [ @{$parsed_sql->{'selectfields'}} ]; }
+		}
+	}
+sub finish
+	{ 1; }
 1;
+
+__END__
+
+=head1 SYNOPSIS
+
+    use DBI;
+    my $dbh = DBI->connect("DBI:XBase:/directory/subdir")
+    				or die $DBI::errstr;
+    my $sth = $dbh->prepare("select MSG from test where ID != 1")
+    				or die $dbh->errstr();
+    $sth->execute() or die $sth->errstr();
+
+    my @data;
+    while (@data = $sth->fetchrow_array())
+		{
+		## further processing
+		}
+
+=head1 DESCRIPTION
+
+DBI compliant driver for module XBase. Please refer to DBI(3)
+documentation for how to actually use the module.
+In the B<connect> call, specify the directory for a database name.
+This is where the DBD::XBase will look for the tables.
+
+The SQL commands currently supported by DBD::XBase include:
+
+=over 4
+
+=item select
+
+    select fields from table [ where condition ]
+
+Fields is a comma separated list of fields or a C<*> for all. The
+C<where> condition specifies which rows will be returned, you can
+compare fields and constants and stack expressions using C<and> or
+C<or>, and also use brackets. Examples:
+
+    select * from salaries where name = "Smith"	
+    select first,last from people where login = "ftp"
+						or uid = 1324
+
+=item delete
+
+    delete from table [ where condition ]
+
+The C<where> condition si the same as for C<select>. Examples:
+
+    delete from jobs		## emties the table
+    delete from jobs where companyid = "ISW"
+
+=item insert
+
+    insert into table [ ( fields ) ] values ( list of values )
+
+Here fields is a comma separated list of fields to set, list of values
+is a list of values to assign. If the fields are not specified, the
+fields in the natural order in the table are set. Example:
+
+    insert into accounts (login, uid) values ("guest", 65534)
+
+=item update
+
+    update table set field = new value [ , set more fields ]
+					[ where condition ]
+
+Example:
+
+    update table set uid = 65534 where login = "guest"
+
+=item create table
+
+    create table table name ( columns specification )
+
+Columns specification is a comma separated list of column names and
+types. Example:
+
+    create table rooms ( roomid int, category int, balcony boolean)
+
+=back
+
+=head1 VERSION
+
+0.063
+
+=head1 AUTHOR
+
+(c) Jan Pazdziora, adelton@fi.muni.cz
+
+=head1 SEE ALSO
+
+perl(1), DBI(3), XBase(3)
+
+=cut
+
